
-- Chapter 10: Query Execution

-- To compare the various ways of viewing query plans, consider the following query:

DECLARE @Country nvarchar(15)
SET @Country = N'USA'
SELECT O.[CustomerId], MAX(O.[Freight]) AS MaxFreight
FROM [Customers] C JOIN [Orders] O
  ON C.[CustomerId] = O.[CustomerId]
WHERE C.[Country] = @Country
GROUP BY O.[CustomerId]
OPTION (OPTIMIZE FOR (@Country = N'UK'));

-- First look at an example of a scan. Consider the following query:

SELECT [OrderId] FROM [Orders] 
WHERE [RequiredDate] = '1998-03-26';

-- Suppose that we have a similar query, but this time the predicate is on 
-- the OrderDate column on which we do have an index:

SELECT [OrderId] FROM [Orders] WHERE [OrderDate] = '1998-02-26';

-- Suppose that you have this schema:

CREATE TABLE T_heap (a int, b int, c int, d int, e int, f int);
CREATE INDEX T_heap_a ON T_heap (a);
CREATE INDEX T_heap_bc ON T_heap (b, c);
CREATE INDEX T_heap_d ON T_heap (d) INCLUDE (e);
CREATE UNIQUE INDEX T_heap_f ON T_heap (f);
CREATE TABLE T_clu (a int, b int, c int, d int, e int, f int);
CREATE UNIQUE CLUSTERED INDEX T_clu_a ON T_clu (a);
CREATE INDEX T_clu_b ON T_clu (b);
CREATE INDEX T_clu_ac ON T_clu (a, c);
CREATE INDEX T_clu_d ON T_clu (d) INCLUDE (e);
CREATE UNIQUE INDEX T_clu_f ON T_clu (f);

-- Look at the following query:

SELECT [OrderId], [CustomerId] FROM [Orders] 
WHERE [OrderDate] = '1998-02-26';

-- Consider this query:

SELECT O.[OrderId]
FROM [Customers] C JOIN [Orders] O 
   ON C.[CustomerId] = O.[CustomerId]
WHERE C.[City] = N'London';

-- Consider the following query, which returns the number of 
-- employees who were hired after each other employee:

SELECT E1.[EmployeeId], COUNT(*)
FROM [Employees] E1 JOIN [Employees] E2
    ON E1.[HireDate] < E2.[HireDate]
GROUP BY E1.[EmployeeId];

-- Now consider the following identical query which has been 
-- rewritten to use a CROSS APPLY:

SELECT E1.[EmployeeId], ECnt.Cnt
FROM [Employees] E1 CROSS APPLY
(
  SELECT COUNT(*) Cnt
  FROM [Employees] E2
  WHERE E1.[HireDate] < E2.[HireDate]
) ECnt;

-- Make the rather unrealistic assumption that two customers with the same name 
-- are indeed the same customer.

CREATE TABLE [Customer1] 
    ([CustomerId] int PRIMARY KEY, [Name] nvarchar(30));
CREATE TABLE [Customer2] 
    ([CustomerId] int PRIMARY KEY, [Name] nvarchar(30));

SELECT C1.[Name], C1.[CustomerId], C2.[CustomerId]
FROM [Customer1] C1 FULL OUTER JOIN [Customer2] C2
    ON C1.[Name] = C2.[Name];

-- The following query simply joins the Orders and Customers tables:

SELECT O.[OrderId], C.[CustomerId], C.[ContactName]
FROM [Orders] O JOIN [Customers] C
    ON O.[CustomerId] = C.[CustomerId];

-- The following query returns a list of orders that shipped 
-- to cities different from the city that we have on file for 
-- the customer who placed the order:

SELECT O.[OrderId], C.[CustomerId], C.[ContactName]
FROM [Orders] O JOIN [Customers] C
    ON O.[CustomerId] = C.[CustomerId] 
       AND O.[ShipCity] <> C.[City]
ORDER BY C.[CustomerId];

-- The following query is nearly identical to the earlier merge 
-- join example except that we select one additional column, 
-- the OrderDate column, from the Orders table:

SELECT O.[OrderId], O.[OrderDate], C.[CustomerId], C.[ContactName]
FROM [Orders] O JOIN [Customers] C
    ON O.[CustomerId] = C.[CustomerId];

-- First consider a trivial example:

SELECT COUNT(*) FROM [Orders];

-- A single-stream aggregate operator can calculate multiple 
-- scalar aggregates at the same time:

SELECT MIN([OrderDate]), MAX([OrderDate]) FROM [Orders];

-- Some aggregates such as AVG are actually calculated from 
-- two other aggregates such as SUM and COUNT:

SELECT AVG([Freight]) FROM [Orders];

-- Although SUM does not need to be computed per se, it still 
-- needs the count:

SELECT SUM([Freight]) FROM [Orders];

-- Consider this query to compute the number of distinct cities 
-- to which orders have been shipped:

SELECT COUNT(DISTINCT [ShipCity]) FROM [Orders];

-- For example, this query gets the same plan as the 
-- above MIN/MAX query without the DISTINCT keyword:

SELECT MIN(DISTINCT [OrderDate]), MAX(DISTINCT [OrderDate]) 
FROM [Orders];

-- For example, the following query is identical to the simple COUNT(*) query 
-- with which this discussion began:

SELECT COUNT(DISTINCT [OrderId]) FROM [Orders];

-- Multiple Distinct. Consider this query:

SELECT COUNT(DISTINCT [ShipAddress]), COUNT(DISTINCT [ShipCity])
FROM [Orders];

-- Consider the following query that counts the number of 
-- orders shipped to each address:

SELECT [ShipAddress], [ShipCity], COUNT(*)
FROM [Orders]
GROUP BY [ShipAddress], [ShipCity];

-- Suppose that the previous query is extended with an ORDER BY clause 
-- that sorts the results on the GROUP BY keys:

SELECT [ShipAddress], [ShipCity], COUNT(*)
FROM [Orders]
GROUP BY [ShipAddress], [ShipCity]
ORDER BY [ShipAddress], [ShipCity];

-- Consider the following query which counts 
-- orders by customer (instead of shipping address):

SELECT [CustomerId], COUNT(*)
FROM [Orders]
GROUP BY [CustomerId];

-- For example, the following,

SELECT DISTINCT [CustomerId] FROM [Orders];

-- Can also be written as follows:

SELECT [CustomerId] FROM [Orders] GROUP BY [CustomerId];

-- Suppose that we want to find the number of distinct customers 
-- served by each employee:

SELECT [EmployeeId], COUNT(DISTINCT [CustomerId])
FROM [Orders]
GROUP BY [EmployeeId];

-- For this example, you create a temporary index on the EmployeeId and 
-- CustomerId columns of the Orders table:

CREATE INDEX [EmployeeCustomer] 
    ON [Orders] (EmployeeId, CustomerId);

-- We need to drop the temporary index before continuing:

DROP INDEX [Orders].[EmployeeCustomer];

-- Suppose that we want to find the total number of orders taken by each employee, 
-- as well as the total number of distinct customers served by each employee:

SELECT [EmployeeId], COUNT(*), COUNT(DISTINCT [CustomerId])
FROM [Orders]
GROUP BY [EmployeeId];

-- Now consider the following essentially identical query, 
-- which groups the Orders table on the ShipCountry column:

SELECT [ShipCountry], COUNT(*)
FROM [Orders]
GROUP BY [ShipCountry];

-- However, suppose we explicitly request a sort using an 
-- ORDER BY clause in the query:

SELECT [ShipCountry], COUNT(*)
FROM [Orders]
GROUP BY [ShipCountry]
ORDER BY [ShipCountry];

-- However, if the preceding query is repeated against the BigOrders table, 
-- the same 21 groups still result:

SELECT [ShipCountry], COUNT(*)
FROM [BigOrders]
GROUP BY [ShipCountry]
ORDER BY [ShipCountry];

-- Suppose that we want just a list of distinct countries to which orders 
-- have been shipped:

SELECT DISTINCT [ShipCountry] FROM [Orders];

-- The Northwind2 database includes a HugeOrders table, which includes the same data 
-- from the original Orders table repeated 25 times.

SELECT [ShipCountry], 
       COUNT(DISTINCT [EmployeeId]), COUNT(DISTINCT [CustomerId])
FROM [HugeOrders]
GROUP BY [ShipCountry];

-- For example, this simple UNION ALL query returns a list of all 
-- employees and customers:

SELECT [FirstName] + N' ' + [LastName], [City], [Country] 
FROM [Employees]
UNION ALL
SELECT [ContactName], [City], [Country] 
FROM [Customers];

-- Now consider the following similar query, which outputs a list of all cities 
-- and countries in which employees and/or customers are located:

SELECT [City], [Country] FROM [Employees]
UNION
SELECT [City], [Country] FROM [Customers];

-- Consider the following query, which combines data from the Orders and BigOrders 
-- tables to generate a list of all countries to which orders have shipped:

SELECT [ShipCountry] FROM [Orders]
UNION
SELECT [ShipCountry] FROM [BigOrders];

-- The following script creates the new table and indexes along with the test query:

SELECT [EmployeeId], 
    [FirstName] + N' ' + [LastName] AS [ContactName],
    [City], [Country]
INTO [NewEmployees]
FROM [Employees];

ALTER TABLE [NewEmployees] 
    ADD CONSTRAINT [PK_NewEmployees] PRIMARY KEY ([EmployeeId]);
CREATE INDEX [ContactName] ON [NewEmployees]([ContactName]);
CREATE INDEX [ContactName] ON [Customers]([ContactName]);

SELECT [ContactName] FROM [NewEmployees]
UNION ALL
SELECT [ContactName] FROM [Customers]
ORDER BY [ContactName];

-- In other words, we want to eliminate duplicates, but we are 
-- not interested in whether the results are sorted.

SELECT [ContactName] FROM [NewEmployees]
UNION
SELECT [ContactName] FROM [Customers];

-- Before continuing, you can drop the extra table and indexes 
-- created just for this example:

DROP TABLE [NewEmployees];
DROP INDEX [Customers].[ContactName];

-- LISTING 10-2 Hash union example

CREATE TABLE [BigTable] 
    ([PK] int PRIMARY KEY, [Dups] int, [Pad] char(1000));
CREATE TABLE [SmallTable] 
    ([PK] int PRIMARY KEY, [NoDups] int UNIQUE, [Pad] char(1000));
SET NOCOUNT ON;
DECLARE @i int;
SET @i = 0;
BEGIN TRAN
WHILE @i < 100000
BEGIN
  INSERT [BigTable] VALUES (@i, 0, NULL);
  SET @i = @i + 1;
  IF @i % 1000 = 0
  BEGIN
    COMMIT TRAN;
    BEGIN TRAN;
  END
END
COMMIT TRAN;
SELECT [Dups], [Pad] FROM [BigTable]
UNION
SELECT [NoDups], [Pad] FROM [SmallTable];

-- Consider the following query with a simple IN list predicate:

SELECT [OrderId]
FROM [Orders]
WHERE [ShipPostalCode] IN (N'05022', N'99362');

-- Now consider the following identical query, which uses 
-- variables in place of constants:

DECLARE @SPC1 nvarchar(20), @SPC2 nvarchar(20);
SELECT @SPC1 = N'05022', @SPC2 = N'99362';

SELECT [OrderId]
FROM [Orders]
WHERE [ShipPostalCode] IN (@SPC1, @SPC2);

-- To answer this question, consider a slightly more complex query:

DECLARE @OD1 datetime, @OD2 datetime;
SELECT @OD1 = '1998-01-01', @OD2 ='1998-01-04';

SELECT [OrderId]
FROM [Orders]
WHERE [OrderDate] BETWEEN @OD1 AND DATEADD(day, 6, @OD1)
   OR [OrderDate] BETWEEN @OD2 AND DATEADD(day, 6, @OD2);

-- Now consider a slightly different query that ORs predicates on 
-- two different columns:

SELECT [OrderId]
FROM [Orders]
WHERE [OrderDate] BETWEEN '1998-01-01' AND '1998-01-07'
   OR [ShippedDate] BETWEEN '1998-01-01' AND '1998-01-07';

-- The optimizer effectively rewrote the query as a union:

SELECT [OrderId]
FROM [Orders]
WHERE [OrderDate] BETWEEN '1998-01-01' AND '1998-01-07'
UNION
SELECT [OrderId]
FROM [Orders]
WHERE [ShippedDate] BETWEEN '1998-01-01' AND '1998-01-07';

-- Now consider the following nearly identical query:

SELECT [OrderId]
FROM [Orders]
WHERE [OrderDate] = '1998-01-01'
   OR [ShippedDate] = '1998-01-01';

-- Consider the following query, which is identical to the previous query 
-- but selects the OrderDate and ShippedDate columns in addition 
-- to the OrderId column:

SELECT [OrderId], [OrderDate], [ShippedDate]
FROM [BigOrders]
WHERE [OrderDate] = '1998-01-01'
   OR [ShippedDate] = '1998-01-01';

-- Consider the following query:

SELECT [OrderId]
FROM [Orders]
WHERE [OrderDate] = '1998-02-26'
  AND [ShippedDate] = '1998-03-04';

-- The optimizer has effectively rewritten this query as a join 
-- (although the explicit rewrite doesnt get the same plan):

SELECT O1.[OrderId]
FROM [Orders] O1 JOIN [Orders] O2
   ON O1.[OrderId] = O2.[OrderId]
WHERE O1.[OrderDate] = '1998-02-26'
  AND O2.[ShippedDate] = '1998-03-04';

-- Now consider the following query which searches for orders 
-- that match a range of dates:

SELECT [OrderId]
FROM [BigOrders]
WHERE [OrderDate] BETWEEN '1998-02-01' AND '1998-02-04'
  AND [ShippedDate] BETWEEN '1998-02-09' AND '1998-02-12';

-- For example, the following query, which selects the OrderDate 
-- and ShippedDate columns in addition to the OrderId column, 
-- uses the same plan as the similar example that selected just 
-- the OrderId column:

SELECT [OrderId], [OrderDate], [ShippedDate]
FROM [Orders]
WHERE [OrderDate] = '1998-02-26'
  AND [ShippedDate] = '1998-03-04';

-- The following query returns a list of orders where the freight
-- charge exceeds the average freight charge for all orders:

SELECT O1.[OrderId], O1.[Freight]
FROM [Orders] O1
WHERE O1.[Freight] >
(
  SELECT AVG(O2.[Freight])
  FROM [Orders] O2
);

-- Now look at another noncorrelated scalar subquery, this time to find those 
-- orders placed by a specific customer whos selected by name:

SELECT O.[OrderId]
FROM [Orders] O
WHERE O.[CustomerId] =
(
  SELECT C.[CustomerId]
  FROM [Customers] C
  WHERE C.[ContactName] = N'Maria Anders'
)

-- This is the same reason the following query doesnt compile:

SELECT COUNT(*), C.[CustomerId]
FROM [Customers] C
WHERE C.[ContactName] = N'Maria Anders'

-- For example, as long as we are not concerned that there might 
-- be two customers with the same name, the above query can be
-- written as a simple join: 

SELECT O.[OrderId]
FROM [Orders] O JOIN [Customers] C
   ON O.[CustomerId] = C.[CustomerId]
WHERE C.[ContactName] = N'Maria Anders';

-- While writing this query as a simple join is the best option, 
-- lets see what happens if we create a unique index on the 
-- ContactName column:

CREATE UNIQUE INDEX [ContactName] ON [Customers] ([ContactName]);

SELECT O.[OrderId]
FROM [Orders] O
WHERE O.[CustomerId] =
(
  SELECT C.[CustomerId]
  FROM [Customers] C
  WHERE C.[ContactName] = N'Maria Anders'
);

DROP INDEX [Customers].[ContactName];

-- The following query is similar to the first subquery we tried,
-- but this time it returns those orders in which the freight 
-- charge exceeds the average freight charge for all previously 
-- placed orders:

SELECT O1.[OrderId]
FROM [Orders] O1
WHERE O1.[Freight] >
(
  SELECT AVG(O2.[Freight])
  FROM [Orders] O2
  WHERE O2.[OrderDate] < O1.[OrderDate]
);

-- Suppose that we want to find those orders for which the 
-- freight charge exceeds the average freight charge for 
-- all orders placed by the same customer:

SELECT O1.[OrderId], O1.[Freight]
FROM [Orders] O1
WHERE O1.[Freight] >
(
  SELECT AVG(O2.[Freight])
  FROM [Orders] O2
  WHERE O2.[CustomerId] = O1.[CustomerId]
);

-- Finally, suppose that we want to compute the order with the 
-- maximum freight charge placed by each customer:

SELECT O1.[OrderId], O1.[Freight]
FROM [Orders] O1
WHERE O1.[Freight] =
(
  SELECT MAX(O2.[Freight])
  FROM [Orders] O2
  WHERE O2.[CustomerId] = O1.[CustomerId]
);

-- For example, the following query uses a noncorrelated subquery
-- to return orders placed by customers who live in London:

SELECT O.[OrderId]
FROM [Orders] O
WHERE O.[CustomerId] IN
(
  SELECT C.[CustomerId]
  FROM [Customers] C
  WHERE C.[City] = N'London'
);

-- We can easily write the same query using a correlated subquery:

SELECT O.[OrderId]
FROM [Orders] O
WHERE EXISTS
(
  SELECT *
  FROM [Customers] C
  WHERE C.[CustomerId] = O.[CustomerId]
  AND C.[City] = N'London'
);

-- For a more complex example of subquery decorrelation, consider
-- the following query, which outputs a list of orders along 
-- with the freight charge for each order and the average freight
-- charge for all orders by the same customer:

SELECT O1.[OrderId], O1.[Freight],
(
  SELECT AVG(O2.[Freight])
  FROM [Orders] O2
  WHERE O2.[CustomerId] = O1.[CustomerId]
) Avg_Freight
FROM [Orders] O1;

-- To see how these plans work, use the following script to set up 
-- an artificial scenario:

CREATE TABLE [MainTable] 
   ([PK] int PRIMARY KEY, [Col1] int, [Col2] int, [Col3] int);
CREATE TABLE [WhenTable] 
   ([PK] int PRIMARY KEY, [Data] int);
CREATE TABLE [ThenTable] 
   ([PK] int PRIMARY KEY, [Data] int);
CREATE TABLE [ElseTable] 
   ([PK] int PRIMARY KEY, [Data] int);

INSERT [MainTable] VALUES (1, 11, 101, 1001);
INSERT [MainTable] VALUES (2, 12, 102, 1002);
INSERT [WhenTable] VALUES (11, NULL);
INSERT [ThenTable] VALUES (101, 901);
INSERT [ElseTable] VALUES (102, 902);

SELECT M.[PK],
   CASE WHEN EXISTS 
      (SELECT * FROM [WhenTable] W WHERE W.[PK] = M.[Col1])
    THEN (SELECT T.[Data] FROM [ThenTable] T WHERE T.[PK] = M.[Col2])
      ELSE (SELECT E.[Data] 
            FROM [ElseTable] E 
            WHERE E.[PK] = M.[Col3])
   END AS Case_Expr
FROM [MainTable] M;

DROP TABLE [MainTable], [WhenTable], [ThenTable], [ElseTable];

-- The following script creates two tables. Each table has 250,000 rows and, 
-- thanks to the fixed-length char(200) column, well over 6,500 pages.

CREATE TABLE [HugeTable1]
(
  [Key] int,
  [Data] int,
  [Pad] char(200),
  CONSTRAINT [PK1] PRIMARY KEY ([Key])
);

SET NOCOUNT ON;
DECLARE @i int;

BEGIN TRAN;
SET @i = 0;
WHILE @i < 250000
BEGIN
  INSERT [HugeTable1] VALUES(@i, @i, NULL);
  SET @i = @i + 1;
  IF @i % 1000 = 0
  BEGIN
    COMMIT TRAN;
    BEGIN TRAN;
  END
END
COMMIT TRAN;

SELECT [Key], [Data], [Pad]
  INTO [HugeTable2]
FROM [HugeTable1];

ALTER TABLE [HugeTable2]
  ADD CONSTRAINT [PK2] PRIMARY KEY ([Key]);

-- Now try the simplest possible query:

SELECT [Key], [Data]
FROM [HugeTable1];

-- Now suppose we add a fairly selective predicate to the query:

SELECT [Key], [Data]
FROM [HugeTable1]
WHERE [Data] < 1000;

-- Now observe what happens if we add an ORDER BY clause to the query:

SELECT [Key], [Data]
FROM [HugeTable1]
WHERE [Data] < 1000
ORDER BY [Key];

-- Consider this query:

SELECT MIN([Data]) FROM [HugeTable1]

-- Chapters 11 and 12 discuss this and other hints in more detail.

SELECT MIN(T1.[Key] + T2.[Key])
FROM [HugeTable1] T1 CROSS JOIN [HugeTable2] T2
OPTION (MAXDOP 1);

-- Here is a simple example of a parallel nested loops join:

SELECT T1.[Key], T1.[Data], T2.[Data]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Key] = T2.[Key]
WHERE T1.[Data] < 500

-- Here is one such example:

SELECT T1_Top.[Key], T1_Top.[Data], T2.[Data]
FROM
(
SELECT TOP 100 T1.[Key], T1.[Data]
FROM [HugeTable1] T1
ORDER BY T1.[Data]
) T1_Top,
[HugeTable2] T2
WHERE T1_Top.[Key] = T2.[Key]

-- Consider this query:

SELECT T1.[Key], T1.[Data], T2.[Key]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Data] = T2.[Data]
WHERE T1.[Key] = 0

-- Consider the following query:

SELECT T1.[Key], T1.[Data], T2.[Key]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Key] = T2.[Data]
ORDER BY T1.[Key]

-- For an example of a parallel hash join, consider the following 
-- simple query:

SELECT T1.[Key], T1.[Data], T2.[Key]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Data] = T2.[Data]

-- For example, the following query includes a very selective predicate:

SELECT T1.[Key], T1.[Data], T2.[Key]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Data] = T2.[Data]
WHERE T1.[Key] < 100

-- Now, look at the following query:

SELECT T1.[Key], T1.[Data], T2.[Key]
FROM [HugeTable1] T1 JOIN [HugeTable2] T2
ON T1.[Data] = T2.[Data]
WHERE T1.[Key] < 10000

-- This statement is guaranteed to violate the foreign key 
-- constraint on the ShipVia column and will fail.

UPDATE [Orders]
SET [ShipVia] = 4
WHERE [ShipCity] = N'London';

-- Run the following query:

USE AdventureWorksDW2012;
GO
SELECT *
FROM dbo.FactInternetSales AS f
JOIN dbo.DimProduct AS p ON f.ProductKey = p.ProductKey
JOIN dbo.DimCustomer AS c ON f.CustomerKey = c.CustomerKey
WHERE p.ListPrice > 50 AND c.Gender = 'M';

-- The first choice is the DBCC OPTIMIZER_WHATIF statement that you can use 
-- with the CPUs parameter, as in the following example to simulate a system 
-- with eight processors:

DBCC OPTIMIZER_WHATIF(CPUs, 8)

-- This command affects only your current session that you can reset to 
-- the state it was before by using the ResetAll parameter.

DBCC OPTIMIZER_WHATIF(ResetAll)

-- Keep in mind that all these parameters are case sensitive.

DBCC OPTIMIZER_WHATIF(Status)

-- Start by creating a columnstore index on the FactInternetSales fact table 
-- by running the following statements:

USE AdventureWorksDW2012;
GO
CREATE NONCLUSTERED COLUMNSTORE INDEX csi_FactInternetSales
ON dbo.FactInternetSales
(
ProductKey,
OrderDateKey,
DueDateKey,
ShipDateKey,
CustomerKey,
PromotionKey,
CurrencyKey,
SalesTerritoryKey,
SalesOrderNumber,
SalesOrderLineNumber,
RevisionNumber,
OrderQuantity,
UnitPrice,
ExtendedAmount,
UnitPriceDiscountPct,
DiscountAmount,
ProductStandardCost,
TotalProductCost,
SalesAmount,
TaxAmt,
Freight,
CarrierTrackingNumber,
CustomerPONumber,
OrderDate,
DueDate,
ShipDate
);

-- The following query is joining the FactInternetSale fact table with the 
-- DimDate dimension table, grouping on MonthNumberOfYear and aggregating data 
-- on the SalesAmount column to get the total of sales by month for the 
-- calendar year 2005:

SELECT d.MonthNumberOfYear,
SUM(SalesAmount) AS TotalSales
FROM dbo.FactInternetSales AS f
JOIN dbo.DimDate AS d
ON f.OrderDateKey = d.DateKey
WHERE CalendarYear = 2005
GROUP BY d.MonthNumberOfYear;

-- You can drop the columnstore index before continuing.

DROP INDEX csi_FactInternetSales ON dbo.FactInternetSales;

-- For example, you can run the following statement to create a copy of 
-- the FactInternetSales table:

SELECT * INTO dbo.FactInternetSalesCopy
FROM dbo.FactInternetSales;

-- Run the following statement:

UPDATE STATISTICS FactInternetSalesCopy WITH ROWCOUNT = 10000000, 
PAGECOUNT = 1000000;

-- After you finish testing with the FactInternetSalesCopy table, you can drop it
-- by running the following statement:

DROP TABLE dbo.FactInternetSalesCopy;

-- Update your query to use this table as shown here:

SELECT d.MonthNumberOfYear,
SUM(SalesAmount) AS TotalSales
FROM dbo.FactInternetSalesBig AS f
JOIN dbo.DimDate AS d
ON f.OrderDateKey = d.DateKey
WHERE CalendarYear = 2005
GROUP BY d.MonthNumberOfYear;

-- Then we create a view that concatenates both the updatable table and 
-- the non-updateable fact table:

CREATE VIEW vFactInternetSales
AS
SELECT * FROM dbo.FactInternetSales
UNION ALL
SELECT * FROM dbo.FactInternetSalesBig;

-- Now, we are ready to run the following star join query using 
-- the view just created:

SELECT d.MonthNumberOfYear,
SUM(SalesAmount) AS TotalSales
FROM dbo.vFactInternetSales AS f
JOIN dbo.DimDate AS d
ON f.OrderDateKey = d.DateKey
WHERE CalendarYear = 2005
GROUP BY d.MonthNumberOfYear;

-- You can use the new IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX hint to ask 
-- the query optimizer to avoid using any columnstore index, as shown next:

SELECT d.MonthNumberOfYear,
SUM(SalesAmount) AS TotalSales
FROM dbo.FactInternetSalesBig AS f
JOIN dbo.DimDate AS d
ON f.OrderDateKey = d.DateKey
WHERE CalendarYear = 2005
GROUP BY d.MonthNumberOfYear
OPTION (IGNORE_NONCLUSTERED_COLUMNSTORE_INDEX);

-- You can also force the use of a columnstore index by using the existing 
-- INDEX hint, as shown in the following example:

SELECT d.MonthNumberOfYear,
SUM(SalesAmount) AS TotalSales
FROM dbo.FactInternetSalesBig AS f WITH (INDEX(csi_FactInternetSalesBig))
JOIN dbo.DimDate AS d
ON f.OrderDateKey = d.DateKey
WHERE CalendarYear = 2005
GROUP BY d.MonthNumberOfYear;

