
-- Chapter 13: Locking and Concurrency

-- Page 772

-- Table 13-1 Two simultaneous transactions in snapshot isolation that cannot be run serially 

USE pubs  
SET TRANSACTION ISOLATION LEVEL SNAPSHOT
DECLARE @price money 
BEGIN TRAN

SELECT @price = price  
FROM titles  
WHERE title_id = 'BU1032'

UPDATE titles  
SET price = @price 
WHERE title_id = 'PS7777'

COMMIT TRAN

USE pubs 
SET TRANSACTION ISOLATION LEVEL SNAPSHOT
DECLARE @price money 
BEGIN TRAN

SELECT @price = price  
FROM titles  
WHERE title_id = 'PS7777'

UPDATE titles  
SET price = @price 
WHERE title_id = 'BU1032'

COMMIT TRAN


-- Page 779

-- We can see those locks by first running this code:

USE Adventureworks2012; 
BEGIN TRAN 
UPDATE  Sales.SalesOrderHeader 
SET ShipDate = ShipDate + 1 
WHERE SalesOrderID = 43666;



-- Page 780

-- You can look at those locks using the sys.dm_tran_locks view:

SELECT resource_type, resource_description,  
        resource_associated_entity_id, request_mode, request_status 
FROM sys.dm_tran_locks 
WHERE resource_associated_entity_id > 0





-- Page 784

-- Listing 13-1 Creating a view to return all locks in the current database

CREATE VIEW DBlocks AS 
SELECT request_session_id as spid,  
    db_name(resource_database_id) as dbname,  
    CASE  
   WHEN resource_type = 'OBJECT' THEN  
         object_name(resource_associated_entity_id) 
      WHEN resource_associated_entity_id = 0 THEN 'n/a' 
   ELSE object_name(p.object_id)  
    END as entity_name, index_id, 
       resource_type as resource,  
       resource_description as description,  
       request_mode as mode, request_status as status 
FROM sys.dm_tran_locks t LEFT JOIN sys.partitions p 
   ON p.hobt_id = t.resource_associated_entity_id 
WHERE resource_database_id = db_id();



-- Page 789

-- Listing 13-2 SELECT with default isolation level
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
BEGIN TRAN
SELECT * FROM Production.Product
WHERE Name = 'Reflector';
SELECT * FROM DBlocks WHERE spid = @@spid;
COMMIT TRAN


-- Listing  13-3 SELECT with Repeatable Read isolation level
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL REPEATABLE READ;
BEGIN TRAN
SELECT * FROM Production.Product
WHERE Name LIKE 'Racing Socks%';
SELECT * FROM DBlocks
WHERE spid = @@spid  
WHERE spid = @@spid 
AND entity_name = 'Product'; 
COMMIT TRAN



-- Page 790


-- Listing 13-4 SELECT with Serializable isolation level

USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
BEGIN TRAN
SELECT * FROM Production.Product
WHERE Name LIKE 'Racing Socks%';
SELECT * FROM DBlocks
WHERE spid = @@spid
AND entity_name = 'Product';
COMMIT TRAN


-- Page 791


-- Listing 13-5 Update operations
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
BEGIN TRAN
UPDATE Production.Product
SET ListPrice = ListPrice * 0.6
WHERE Name LIKE 'Racing Socks%';
SELECT * FROM DBlocks
WHERE spid = @@spid
AND entity_name = 'Product';
COMMIT TRAN


-- Page 792

-- Listing 13-6 Update with Serializable isolation level using an index
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
BEGIN TRAN
UPDATE Production.Product
SET ListPrice = ListPrice * 0.6
WHERE Name LIKE 'Racing Socks%'; 
SELECT * FROM DBlocks  
WHERE spid = @@spid 
AND entity_name = 'Product'; 
COMMIT TRAN



-- Listing 13-7 Update with Serializable isolation not using an index
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL SERIALIZABLE;
BEGIN TRAN
UPDATE Production.Product
SET ListPrice = ListPrice * 0.6
WHERE Color = 'White';
SELECT * FROM DBlocks
WHERE spid = @@spid
AND entity_name = 'Product';
COMMIT TRAN


-- Page 793

-- Listing 13-8 Creating a table
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED;
BEGIN TRAN
SELECT *
INTO newProducts
FROM Production.Product
WHERE ListPrice between 1 and 10;
SELECT * FROM DBlocks
WHERE spid = @@spid;
COMMIT TRAN


-- Page 794

-- Listing 13-9 Row locks
 
USE AdventureWorks2012;
SET TRANSACTION ISOLATION LEVEL READ COMMITTED
BEGIN TRAN
UPDATE newProducts
SET ListPrice = 5.99
WHERE name = 'Road Bottle Cage';
SELECT * FROM DBlocks
WHERE spid = @@spid
AND entity_name = 'newProducts';
COMMIT TRAN



-- Page 828 - 829

-- You can then look at the rsc_bin column in syslockinfo for key locks, page locks, and table locks:

USE AdventureWorks2012
GO 
SET TRANSACTION ISOLATION LEVEL REPEATABLE READ 
GO 
BEGIN TRAN 
SELECT * FROM Person.Person  
WHERE BusinessEntityID = 249 
GO 
SELECT rsc_bin, rsc_type  
FROM master..syslockinfo 
WHERE rsc_type IN (5,6,7) 
GO




-- Page 802

-- The following example creates a clustered and nonclustered index on a 
-- tiny table and then selects the %%lockres%% value for each row, first using the clustered 
-- index and then using the nonclustered index:

CREATE TABLE lockres (c1 int, c2 int);
GO
INSERT INTO lockres VALUES (1,10);
INSERT INTO lockres VALUES (2,20);
INSERT INTO lockres VALUES (3,30);
GO
CREATE UNIQUE CLUSTERED INDEX lockres_ci ON lockres(c1);
CREATE UNIQUE NONCLUSTERED INDEX lockres_nci ON lockres(c2);
GO
SELECT %%lockres%% AS lock_resource, * FROM lockres WITH (INDEX = lockres_ci);
SELECT %%lockres%% AS lock_resource, * FROM lockres WITH (INDEX = lockres_nci);
GO



-- Page 803

-- The value returned looks just like the special value %%physloc%% that was mentioned in 
-- Chapter 6, Table storage:

CREATE TABLE lockres_on_heap (c1 int, c2 int);
GO
INSERT INTO lockres_on_heap VALUES (1,10);
INSERT INTO lockres_on_heap VALUES (2,20);
INSERT INTO lockres_on_heap VALUES (3,30);
GO
SELECT %%lockres%% AS lock_resource, * FROM lockres_on_heap;



-- Page 813

-- Table 13-9 A SELECT Running in RCSI

-- Transaction 1

BEGIN TRAN 
UPDATE Production.Product 
SET ListPrice = 10.00 
WHERE ProductID  = 922; 

COMMIT TRAN


-- Transaction 2

BEGIN TRAN

SELECT ListPrice 
FROM Production.Product 
WHERE ProductID  = 922; 
-- SQL Server returns 8.89 

SELECT ListPrice 
FROM Production.Product 
WHERE ProductID  = 922; 
-- SQL Server returns 10.00

COMMIT TRAN



-- Page 816


-- Table 13-10 A SELECT Running in a SNAPSHOT Transaction

-- Transaction 1

BEGIN TRAN

UPDATE Production.Product 
SET ListPrice = 12.00 
WHERE ProductID = 922;

COMMIT TRAN


-- Transaction 2

SET TRANSACTION ISOLATION 
LEVEL SNAPSHOT

BEGIN TRAN

SELECT ListPrice 
FROM Production.Product 
WHERE ProductID = 922; 
-- SQL Server returns 10.00 
-- This is the beginning of 
-- the transaction

SELECT ListPrice 
FROM Production.Product 
WHERE ProductID = 922; 
-- SQL Server returns 10.00 
-- Return the committed 
-- value as of the beginning 
-- of the transaction

COMMIT TRAN 

SELECT ListPrice 
FROM Production.Product 
WHERE ProductID = 922; 
-- SQL Server returns 12.00



-- Page 818

-- Table 13-13 An Update Conflict in SNAPSHOT Isolation

-- Transaction 1

BEGIN TRAN 
UPDATE Production.ProductInventory 
SET Quantity=Quantity + 200 
WHERE ProductID  = 872;  
-- Quantity is now 524

COMMIT TRAN


-- Transaction 2

SET TRANSACTION ISOLATION  
LEVEL SNAPSHOT

BEGIN TRAN

SELECT Quantity 
FROM Production.ProductInventory 
WHERE ProductID  = 872; 
-- SQL Server returns 324 
-- This is the beginning of 
-- the transaction

UPDATE Production.ProductInventory 
SET Quantity=Quantity + 300 
WHERE ProductID = 872;  
-- Process will block

--  Process receives error 3960



-- Page 820

-- Table 13-14 DDL Inside a SNAPSHOT Transaction

-- Transaction 1

SET TRANSACTION ISOLATION  
LEVEL SNAPSHOT

BEGIN TRAN

SELECT count(*)  
FROM Production.Product 
-- This is the beginning of 
-- the transaction 

CREATE TABLE NewProducts 
( <column definitions>) 
-- This DDL is legal

CREATE INDEX PriceIndex 
   ON Production.Product 
     (ListPrice) 
-- This DDL will generate an  
-- error


-- Transaction 2

BEGIN TRAN 

INSERT Production.Product 
  VALUES (9999, .....) 
 
-- A new row is insert into 
--   the Product table

COMMIT TRAN




-- Table 13-15 Concurrent DDL Outside the SNAPSHOT Transaction 

-- Transaction 1

SET TRANSACTION ISOLATION  
LEVEL SNAPSHOT

BEGIN TRAN 

SELECT TOP 10 * 
FROM Production.Product; 
-- This is the start of  
-- the transaction

SELECT TOP 10 * 
FROM Production.Product; 
-- Succeeds 
-- The ALTER to a different 
--  table does not affect  
--  this transaction

SELECT TOP 10 * FROM Production.Product; 
-- ERROR 


-- Transaction 2

BEGIN TRAN 
ALTER TABLE Purchasing.Vendor 
   ADD notes varchar(1000); 
COMMIT TRAN

BEGIN TRAN 
ALTER TABLE Production.Product 
   ADD LowestPrice money; 
COMMIT TRAN



-- Page 823

-- You might need to close any active transactions currently using AdventureWorks2012: 

USE AdventureWorks2012 
SELECT * INTO NewProduct 
FROM Production.Product; 
GO 
ALTER DATABASE ADVENTUREWORKS2012 SET ALLOW_SNAPSHOT_ISOLATION ON; 
GO 
SELECT name, snapshot_isolation_state_desc,  
      is_read_committed_snapshot_on  
FROM sys.databases  
WHERE name = 'AdventureWorks2012'; 
GO 
SELECT COUNT(*) FROM sys.dm_tran_version_store;
GO


-- So now run this UPDATE statement on the NewProduct table, and look at the version store again:

UPDATE  NewProduct 
SET ListPrice = ListPrice * 1.1; 
GO 
SELECT COUNT(*) FROM sys.dm_tran_version_store; 
GO



-- Page 824

-- The output shows only one of the rows inserted:

CREATE TABLE T1 (T1ID char(1), T1name char(10)) 
GO 
INSERT T1 SELECT 'A', 'aaaaaaaaaa' 
INSERT T1 SELECT 'B', 'bbbbbbbbbb' 
GO 
SELECT allocated_page_file_id, allocated_page_page_id, page_type_desc
FROM sys.dm_db_database_page_allocations
(db_id('AdventureWorks2012'), object_id('T1'), NULL, NULL, 'DETAILED');
DBCC TRACEON (3604);
DBCC PAGE('AdventureWorks2012', 1, xxxxx, 1);



-- Page 825

-- If I update one of these rows, the previous row is written to the version store and 
-- the XSN is reflected in the row versioning information:

UPDATE T1 SET T1name = '2222222222' where T1ID = 'A'; 
GO 
DBCC PAGE('AdventureWorks2012', 1, 6709, 1) 
GO 



-- Page 828 - 829


-- Ill use the AdventureWorks2012 database, which has ALLOW_SNAPSHOT_ISOLATION set to ON, 
-- and Ill create a simple table:

CREATE TABLE t1 
(col1 int primary key, col2 int); 
GO 
INSERT INTO t1 SELECT 1,10; 
INSERT INTO t1 SELECT 2,20; 
INSERT INTO t1 SELECT 3,30;


-- Change the sessions isolation level, start a snapshot transaction, and examine some of the metadata: 

SET TRANSACTION ISOLATION LEVEL SNAPSHOT 
GO 
BEGIN TRAN 
SELECT * FROM t1; 
GO 
select * from sys.dm_tran_current_transaction;  
select * from sys.dm_tran_version_store; 
select * from sys.dm_tran_transactions_snapshot;


-- In another connection (Connection 2), run an update and examine some of the metadata for 
-- the current transaction:

BEGIN TRAN 
 UPDATE T1 SET col2 = 100 
   WHERE col1 = 1; 
SELECT * FROM sys.dm_tran_current_transaction;


-- It is almost identical to the first, but there is an important difference in the metadata results:

SET TRANSACTION ISOLATION LEVEL SNAPSHOT 
GO 
BEGIN TRAN 
SELECT * FROM t1; 
GO 
select * from sys.dm_tran_current_transaction;  
select * from sys.dm_tran_transactions_snapshot;


-- Now lets go back to the first SELECT transaction in Connection 1 and rerun the SELECT statement, 
-- staying in the same transaction:

SELECT * FROM t1;


-- We can examine the sys.dm_tran_active_ snapshot_database_transactions view with this query:

SELECT transaction_sequence_num, commit_sequence_num,  
   is_snapshot, session_id,first_snapshot_sequence_num, 
   max_version_chain_traversed, elapsed_time_seconds 
FROM  sys.dm_tran_active_snapshot_database_transactions;


-- You could actually run the preceding query from any connection; it shows ALL active snapshot 
-- transactions in the SQL Server instance, and because it includes the session_id, you can join it 
-- to sys.dm_exec_sessions to get information about the connection that's running the transaction:

SELECT transaction_sequence_num, commit_sequence_num,  
    is_snapshot, t.session_id,first_snapshot_sequence_num, 
    max_version_chain_traversed, elapsed_time_seconds,  
    host_name, login_name, transaction_isolation_level 
FROM  sys.dm_tran_active_snapshot_database_transactions t 
   JOIN sys.dm_exec_sessions s 
     ON t.session_id = s.session_id;




-- Page 835


-- The following example illustrates the differences between using READPAST, using READUNCOMMITTED, and setting LOCK_TIMEOUT to 0. 
-- All these techniques let you avoid blocking problems, but the behavior is slightly different in each case.


-- 1. In a new query window, execute the following batch to lock one row in the HumanResources.Department table:

USE AdventureWorks2012;
BEGIN TRAN;
UPDATE HumanResources.Department
SET ModifiedDate = getdate()
WHERE DepartmentID = 1;

-- 2. Open a second connection and execute the following statements:

USE AdventureWorks2012;
SET LOCK_TIMEOUT 0;
SELECT * FROM HumanResources.Department;
SELECT * FROM Sales.SalesPerson;

--3. Open a third connection and execute the following statements:

USE AdventureWorks2012;
SELECT * FROM HumanResources.Department (READPAST);
SELECT * FROM Sales.SalesPerson;

-- 4. Open a fourth connection and execute the following statements:
USE AdventureWorks2012;
SELECT * FROM HumanResources.Department (READUNCOMMITTED);
SELECT * FROM Sales.SalesPerson;






