-- Chapter 4 "Special databases"

-- Page 149 
-- LISTING 4-1 Inspecting the allocation ring buffer

DECLARE @ts_now bigint;
SELECT @ts_now=cpu_ticks/(cpu_ticks/ms_ticks)
FROM sys.dm_os_sys_info;

SELECT record_id,
       DATEADD(ms, -1 * (@ts_now - [timestamp]), GETDATE()) as event_time,
       CASE
           WHEN event = 0 THEN 'Allocation Cache Init'
           WHEN event = 1 THEN 'Allocation Cache Add Entry'
           WHEN event = 2 THEN 'Allocation Cache RMV Entry'
           WHEN event = 3 THEN 'Allocation Cache Reinit'
           WHEN event = 4 THEN 'Allocation Cache Free'
           WHEN event = 5 THEN 'Truncate Allocation Unit'
           WHEN event = 10 THEN 'PFS Alloc Page'
           WHEN event = 11 THEN 'PFS Dealloc Page'
           WHEN event = 20 THEN 'IAM Set Bit'
           WHEN event = 21 THEN 'IAM Clear Bit'
           WHEN event = 22 THEN 'GAM Set Bit'
           WHEN event = 23 THEN 'GAM Clear Bit'
           WHEN event = 24 THEN 'SGAM Set Bit'
           WHEN event = 25 THEN 'SGAM Clear Bit'
           WHEN event = 26 THEN 'SGAM Set Bit NX'
           WHEN event = 27 THEN 'SGAM Clear Bit NX'
           WHEN event = 28 THEN 'GAM Zap Extent'
           WHEN event = 40 THEN 'Format IAM Page'
           WHEN event = 41 THEN 'Format Page'
           WHEN event = 42 THEN 'Reassign IAM Page'
           WHEN event = 50 THEN 'Worktable Cache Add IAM'
           WHEN event = 51 THEN 'Worktable Cache Add Page'
           WHEN event = 52 THEN 'Worktable Cache RMV IAM'
           WHEN event = 53 THEN 'Worktable Cache RMV Page'
           WHEN event = 61 THEN 'IAM Cache Destroy'
           WHEN event = 62 THEN 'IAM Cache Add Page'
           WHEN event = 63 THEN 'IAM Cache Refresh Requested'
           ELSE 'Unknown Event'
        END as event_name,
        session_id as s_id,
        page_id,
        allocation_unit_id as au_id
    FROM
        (
         SELECT
           xml_record.value('(./Record/@id)[1]', 'int') AS record_id,
           xml_record.value('(./Record/SpaceMgr/Event)[1]', 'int') AS event,
           xml_record.value('(./Record/SpaceMgr/SpId)[1]', 'int') AS session_id,
           xml_record.value('(./Record/SpaceMgr/PageId)[1]', 'varchar(100)') AS page_id,
           xml_record.value('(./Record/SpaceMgr/AuId)[1]', 'varchar(100)') AS allocation_unit_id,
           timestamp
           FROM
               (
                 SELECT timestamp, convert(xml, record) as xml_record
                 FROM sys.dm_os_ring_buffers
                 WHERE ring_buffer_type = N'RING_BUFFER_SPACEMGR_TRACE'
                ) AS the_record
         ) AS ring_buffer_record
    -- WHERE session_id = 52
    ORDER BY record_id;


-- Page 151

-- To detect DML contention, you can use any of the following queries, which look for PAGELATCH waits on -- tempdb, as indicated by the database ID 2 and the waitresource and resource_description columns.

SELECT * FROM sysprocesses
WHERE lastwaittype LIKE 'PAGE%LATCH%' AND waitresource LIKE '2:%';

SELECT * from sys.dm_os_waiting_tasks
WHERE wait_type LIKE 'PAGE%LATCH%'
AND resource_description LIKE '2:%';


-- Page 153 - 154

-- You can try the following exercise, which uses these three DMVs to monitor the number of pages allocated 
-- and deallocated on internal objects at the database data file, session, and task level, respectively. 
-- Open a query window in Management Studio and execute the following code:

USE AdventureWorks2012;
GO
CREATE TABLE dbo.SalesOrderDetail
(
  SalesOrderID int,
  SalesOrderDetailID int,
  CarrierTrackingNumber nvarchar(25),
  OrderQty smallint,
  ProductID int,
  SpecialOfferID int,
  UnitPrice money,
  UnitPriceDiscount money,
  LineTotal numeric(38, 6),
  rowguid uniqueidentifier,
  ModifiedDate datetime);
GO

INSERT INTO dbo.SalesOrderDetail
  SELECT * FROM Sales.SalesOrderDetail;
GO 80

-- Open a second window and use the following code. To help on filtering the output, replace the 
-- session_id value with the corresponding value of the session used by the first window 
-- as shown in Management Studio.

USE tempdb;
GO

SELECT * FROM sys.dm_db_file_space_usage;

SELECT * FROM sys.dm_db_task_space_usage
WHERE session_id = 52;

SELECT * FROM sys.dm_db_session_space_usage
WHERE session_id = 52;

-- Go back to the first window and be ready to run the following statement:

CREATE CLUSTERED INDEX PK_SalesOrderID
 ON dbo.SalesOrderDetail(SalesOrderID, SalesOrderDetailID)
 WITH (SORT_IN_TEMPDB = ON);


-- Page 156

-- Here is the syntax to create a snapshot of the AdventureWorks2012 database, putting the snapshot files 
-- in the C:\Data directory:

CREATE DATABASE AdventureWorks_snapshot ON
 ( NAME = N'AdventureWorks2012_Data',
   FILENAME = N'C:\Data\AdventureWorks_snapshot.mdf'
 )
 AS SNAPSHOT OF AdventureWorks2012;


-- Page 166 - 167

-- Create the new database by running the following statement:
CREATE DATABASE Test COLLATE Finnish_Swedish_CI_AS;

--Then create a regular and a temporary table:
USE Test;
GO

CREATE TABLE table1 (col1 varchar(40));
GO

CREATE TABLE #table2 (col1 varchar(40));
GO


-- The regular table has the same collation as the hosting database, whereas the temporary  
-- table uses the collation of tempdb. Trying to join both tables results in an error:

SELECT * FROM table1 t1 JOIN #table2 t2
  ON t1.col1 = t2.col1;


-- You can run the following code to verify that, in fact, the collations for both tables are different:

USE Test;
GO

SELECT name, collation_name
FROM sys.columns
WHERE name = 'col1';

USE tempdb;
GO

SELECT name, collation_name
FROM sys.columns
WHERE name = 'col1';

-- The first SELECT returns Finnish_Swedish_CI_AS; the second shows SQL_Latin1_General_CP1_CI_AS. 
-- This is no longer a problem with contained databases in which the temporary tables are created 
-- using the same collation as the contained database. To test it, drop the temporary table, 
-- configure Test as a contained database, and create a new temporary table:

DROP TABLE #table2;
GO

USE master;
GO

ALTER DATABASE Test SET CONTAINMENT = PARTIAL;
GO

USE Test;
GO

CREATE TABLE #table2 (col1 varchar(40));
GO


-- Page 168

-- The following example using the sys.dm_db_uncontained_entities DMV creates a few uncontained objects: 
-- a user mapped to a login, a stored procedure referencing a table in AdventureWorks2012, 
-- a stored procedure referencing a non-existing object, and a view using an instance-level system view:

CREATE LOGIN User1 WITH PASSWORD = 'ab@3cd';
GO

CREATE USER User1 FOR LOGIN User1;
GO

CREATE PROCEDURE proc1;
AS
  SELECT * FROM AdventureWorks2012.Sales.SalesOrderDetail;
GO

CREATE PROCEDURE proc2;
AS
  SELECT * FROM non_existing_table;
GO

CREATE VIEW view1;
AS
  SELECT * FROM sys.linked_logins;
GO

-- Next, run the following query:

SELECT class_desc, major_id, statement_type as s_type, feature_name, feature_type_name
FROM sys.dm_db_uncontained_entities;
GO
