-- Chapter 5 "Logging and recovery"

-- Pages 181 - 182

--The following code creates a table called sp_LOGINFO that can hold the output of DBCC LOGINFO. 
-- Because the table is created in the master database and starts with the characters sp_, 
-- it can be accessed and modified in any database.

USE master;
GO

IF EXISTS (SELECT 1 FROM sys.tables
            WHERE name = 'sp_LOGINFO')
DROP TABLE sp_loginfo;
GO

CREATE TABLE sp_loginfo
(RecoveryUniteID int,
  FileId tinyint,
  FileSize bigint,
  StartOffset bigint,
  FSeqNo int,
  Status tinyint,
  Parity tinyint,
  CreateLSN numeric(25,0) );
GO


-- The following code creates a new database called TWO_LOGS and then copies a large table from the -- AdventureWorks2012 sample database into TWO_LOGS, causing the log to grow

USE Master
GO

IF EXISTS (SELECT * FROM sys.databases
           WHERE name = 'TWO_LOGS')
    DROP DATABASE TWO_LOGS;
GO

CREATE DATABASE TWO_LOGS
ON PRIMARY
(NAME = Data ,
  FILENAME = 
    'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\TWO_LOGS.mdf'
 , SIZE = 100 MB)
LOG ON
(NAME = TWO_LOGS1,
  FILENAME =
    'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\TWO_LOGS1.ldf'
   , SIZE = 5 MB
   , MAXSIZE = 2 GB),
(NAME = TWO_LOGS2,
      FILENAME =
'C:\Program Files\Microsoft SQL Server\MSSQL11.MSSQLSERVER\MSSQL\DATA\TWO_LOGS2.ldf'
   , SIZE = 5 MB);
GO

-- If you run DBCC LOGINFO, notice that it returns VLFs sorted by FileID and, initially, 
-- the file sequential number values (FSeqNo) are also in order:

USE TWO_LOGS
GO
DBCC LOGINFO;
GO


-- Now you can insert some rows into the database by copying from another table:
SELECT * INTO Orders
  FROM AdventureWorks2012.Sales.SalesOrderDetail;
GO

-- If you run DBCC LOGINFO again, notice that after the SELECT INTO operation, 
-- even though each FileID has many more rows, the output is still sorted by FileID, 
-- and FSeqNo values arent related at all. Instead, you can save the output of DBCC LOGINFO 
-- in the sp_loginfo table and sort it by FSeqNo:

TRUNCATE TABLE sp_LOGINFO;
INSERT INTO sp_LOGINFO
   EXEC ('DBCC LOGINFO');
GO

-- Unused VLFs have a Status of 0, so the CASE forces those to the end

SELECT FileId, StartOffset, FSeqNo, Status, CreateLSN FROM sp_LOGINFO
ORDER BY CASE FSeqNo WHEN 0 THEN 9999999 ELSE FSeqNo END;
GO


-- Page 185 LISTING 5-2 Observing a database in auto truncate mode

USE master;
IF EXISTS (SELECT * FROM sys.databases WHERE name = 'newdb')
    DROP DATABASE newdb;
GO

CREATE DATABASE newdb;
GO

USE newdb;

-- First look at the VLFs for the newdb database
DBCC LOGINFO;

-- Now verify that newdb is in auto truncate mode
SELECT last_log_backup_lsn
FROM master.sys.database_recovery_status
WHERE database_id = db_id(' newdb ');
GO

CREATE TABLE newtable (a int);
GO
INSERT INTO newtable VALUES (10);
INSERT INTO newtable VALUES (20);
INSERT INTO newtable VALUES (30);
GO
SET NOCOUNT ON
DECLARE @counter int;
SET @counter = 1 ;
WHILE @counter < 1000 BEGIN
  UPDATE newtable SET a = a + 1;
  SET @counter = @counter + 1;
END;
GO


-- Page 188

-- You can use the dynamic management view sys.dm_os_performance_counters and 
-- retrieve the percentage full for each databases log:

SELECT instance_name as [Database], cntr_value as "LogFullPct"
FROM sys.dm_os_performance_counters
WHERE counter_name LIKE 'Percent Log Used%'
  AND instance_name not in ('_Total', 'mssqlsystemresource')
  AND cntr_value > 0;
GO


