-- Chapter 7 "Indexes: internals and management"

-- Page 304
-- LISTING 7-1 Using variables to specify the object with 
--  sys.dm_db_index_physical_stats
DECLARE @db_id SMALLINT;
DECLARE @object_id INT;
SET @db_id = DB_ID (N'AdventureWorks2012');
SET @object_id = OBJECT_ID (N'AdventureWorks2012.Person.Address');
IF (@db_id IS NULL OR @object_id IS NULL)
  BEGIN
    IF @db_id IS NULL
    BEGIN
      PRINT N'Invalid database';
    END;
    ELSE IF @object_id IS NULL
    BEGIN
      PRINT N'Invalid object';
    END
END
ELSE
SELECT * FROM sys.dm_db_index_physical_stats (@db_id, @object_id, NULL, NULL, NULL);
GO

-- Page 318
-- To investigate the Employee table further, use sys.dm_db_index_physical_stats 
-- to determine the number of pages within the table, as well as the number of levels
-- within the indexes. You can confirm the index structures using the DMV 
-- to see the number of levels as well as the number of pages within each level:
SELECT index_depth AS D
, index_level AS L
, record_count AS 'Count'
, page_count AS PgCnt
, avg_page_space_used_in_percent AS 'PgPercentFull'
, min_record_size_in_bytes AS 'MinLen'
, max_record_size_in_bytes AS 'MaxLen'
, avg_record_size_in_bytes AS 'AvgLen'
FROM sys.dm_db_index_physical_stats
(DB_ID ('IndexInternals')
, OBJECT_ID ('IndexInternals.dbo.Employee')
, 1, NULL, 'DETAILED');
GO

-- Page 320
-- LISTING 7-2 Listing the pages from the IndexInternals.dbo.Employee table
SELECT page_level as Level, allocated_page_file_id as PageFID,
       allocated_page_page_id as PagePID,
       previous_page_file_id as PrevPageFID, previous_page_page_id as PrevPagPID,
       next_page_file_id as NextPageFID, next_page_page_id NextPagePID
FROM sys.dm_db_database_page_allocations(db_id('IndexInternals'),
      object_id('dbo.Employee'), 1, null, 'DETAILED')
ORDER BY page_level DESC, previous_page_page_id;
GO


-- Pages 323 - 324
-- LISTING 7-3 Examining the size of a nonclustered index on a heap
SELECT index_depth AS D
, index_level AS L
, record_count AS 'Count'
, page_count AS PgCnt
, avg_page_space_used_in_percent AS 'PgPercentFull'
, min_record_size_in_bytes AS 'MinLen'
, max_record_size_in_bytes AS 'MaxLen'
, avg_record_size_in_bytes AS 'AvgLen'
FROM sys.dm_db_index_physical_stats
  (DB_ID ('IndexInternals'), OBJECT_ID ('IndexInternals.dbo.EmployeeHeap')
   , 2, NULL, 'DETAILED');
GO

-- Page 325
-- The following function converts the RID to the FileID:PageID:SlotNumber format
CREATE FUNCTION convert_RIDs (@rid BINARY(8))
  RETURNS VARCHAR(30)
AS
  BEGIN
  RETURN (
    CONVERT (VARCHAR(5),
      CONVERT(INT, SUBSTRING(@rid, 6, 1)
       + SUBSTRING(@rid, 5, 1)) )
    + ':' +
    CONVERT(VARCHAR(10),
      CONVERT(INT, SUBSTRING(@rid, 4, 1)
      + SUBSTRING(@rid, 3, 1)
      + SUBSTRING(@rid, 2, 1)
      + SUBSTRING(@rid, 1, 1)) )
    + ':' +
    CONVERT(VARCHAR(5),
      CONVERT(INT, SUBSTRING(@rid, 8, 1)
      + SUBSTRING(@rid, 7, 1)) ) )
  END;
GO


-- Page 328
-- Now use that value to look at the information about the index levels:
SELECT index_depth AS D
, index_level AS L
, record_count AS 'Count'
, page_count AS PgCnt
, avg_page_space_used_in_percent AS 'PgPercentFull'
, min_record_size_in_bytes AS 'MinLen'
, max_record_size_in_bytes AS 'MaxLen'
, avg_record_size_in_bytes AS 'AvgLen'
FROM sys.dm_db_index_physical_stats
(DB_ID ('IndexInternals'), OBJECT_ID ('IndexInternals.dbo.Employee')
         , 2, NULL, 'DETAILED');
GO


-- Page 329
-- To see the data stored more specifically, you can use sys.dm_db_database_page_allocations 
-- to review the leaf-level pages of this index:
SELECT page_level as Level, allocated_page_file_id as PageFID,
  allocated_page_page_id as PagePID,
  previous_page_file_id as PrevPageFID, previous_page_page_id as PrevPagPID,
  next_page_file_id as NextPageFID, next_page_page_id NextPagePID
FROM sys.dm_db_database_page_allocations(db_id('IndexInternals'),
      object_id('dbo.Employee'), 2, null, 'DETAILED')
ORDER BY page_level DESC, previous_page_page_id;
GO


-- Page 330 - 331
-- LISTING 7-4 Comparing the size of the leaf-level rows of three indexes
CREATE NONCLUSTERED INDEX TestTreeStructure ON Employee (SSN);
GO
CREATE UNIQUE NONCLUSTERED INDEX TestTreeStructureUnique1 ON Employee (SSN);
GO
CREATE UNIQUE NONCLUSTERED INDEX TestTreeStructureUnique2 ON Employee (SSN, EmployeeID);
GO
SELECT si.[name] AS iname
, index_depth AS D
, index_level AS L
, record_count AS 'Count'
, page_count AS PgCnt
, avg_page_space_used_in_percent AS 'PgPercentFull'
, min_record_size_in_bytes AS 'MinLen'
, max_record_size_in_bytes AS 'MaxLen'
, avg_record_size_in_bytes AS 'AvgLen'
FROM sys.dm_db_index_physical_stats
(DB_ID ('IndexInternals')
, OBJECT_ID ('IndexInternals.dbo.Employee')
, NULL, NULL, 'DETAILED') ps
INNER JOIN sys.indexes si
ON ps.[object_id] = si.[object_id]
AND ps.[index_id] = si.[index_id]
WHERE ps.[index_id] > 2;
GO


-- Page 339
-- The first step in building an index on a view is to create the view itself. 
-- Heres an example from the AdventureWorks2012 database:
USE AdventureWorks2012;
GO
CREATE VIEW Vdiscount1
WITH SCHEMABINDING
AS SELECT SUM (UnitPrice*OrderQty) AS SumPrice
, SUM (UnitPrice * OrderQty * (1.00 - UnitPriceDiscount))
AS SumDiscountPrice
, COUNT_BIG (*) AS Count
, ProductID
FROM Sales.SalesOrderDetail
GROUP BY ProductID;

-- Notice the WITH SCHEMABINDING clause and the specification of the 
-- schema name (Sales) for the table. At this point, its a normal view, 
-- a stored SELECT statement that uses no storage space. In fact, if you 
-- look at the data in sys.dm_db_partition_stats for this view, notice 
-- that no rows are returned:
SELECT si.name AS index_name, ps.used_page_count, ps.reserved_page_count, ps.row_count
FROM sys.dm_db_partition_stats AS ps
    JOIN sys.indexes AS si
      ON ps.[object_id] = si.[object_id]
WHERE ps.[object_id] = OBJECT_ID ('dbo.Vdiscount1');
GO


-- Page 344 - 345
-- LISTING 7-5 Examining a page after a split
USE AdventureWorks2012;
GO
CREATE TABLE bigrows
(
  a int primary key,
  b varchar(1600)
);
GO
/* Insert five rows into the table */
INSERT INTO bigrows
  VALUES (5, REPLICATE('a', 1600));
INSERT INTO bigrows
  VALUES (10, replicate('b', 1600));
INSERT INTO bigrows
  VALUES (15, replicate('c', 1600));
INSERT INTO bigrows
  VALUES (20, replicate('d', 1600));
INSERT INTO bigrows
  VALUES (25, replicate('e', 1600));
GO

SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
FROM sys.dm_db_database_page_allocations(db_id('AdventureWorks2012'),
     object_id('dbo.bigrows'), null, null, 'DETAILED')
WHERE page_type = 1;;
GO


-- Page 347
-- LISTING 7-6 Deleting a row from a heap
USE AdventureWorks2012;
GO
IF object_id('dbo.smallrows') IS NOT NULL
   DROP TABLE dbo.smallrows;
GO
CREATE TABLE dbo.smallrows
(
  a int identity,
  b char(10)
);
GO
INSERT INTO dbo.smallrows VALUES ('row 1');
INSERT INTO dbo.smallrows VALUES ('row 2');
INSERT INTO dbo.smallrows VALUES ('row 3');
INSERT INTO dbo.smallrows VALUES ('row 4');
INSERT INTO dbo.smallrows VALUES ('row 5');
GO
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
FROM sys.dm_db_database_page_allocations(db_id('AdventureWorks2012'),
      object_id('dbo.smallrows'), null, null, 'DETAILED');
WHERE page_type = 1;



-- Page 351
-- LISTING 7-9 Deleting a row from a table with a clustered index
USE AdventureWorks2012;
GO
IF object_id('dbo.smallrows') IS NOT NULL
  DROP TABLE dbo.smallrows;
GO
CREATE TABLE dbo.smallrows
(
  a int IDENTITY PRIMARY KEY,
  b char(10)
);
GO
INSERT INTO dbo.smallrows VALUES ('row 1');
INSERT INTO dbo.smallrows VALUES ('row 2');
INSERT INTO dbo.smallrows VALUES ('row 3');
INSERT INTO dbo.smallrows VALUES ('row 4');
INSERT INTO dbo.smallrows VALUES ('row 5');
GO
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
FROM sys.dm_db_database_page_allocations(db_id('AdventureWorks2012'),
    object_id('dbo.smallrows'), null, null, 'DETAILED');
GO


-- Page 355
LISTING 7-11 Creating a forwarding stub
USE AdventureWorks2012;
GO
IF object_id('dbo.bigrows') IS NOT NULL DROP TABLE bigrows;
GO
CREATE TABLE dbo.bigrows
( a int IDENTITY ,
  b varchar(1600),
  c varchar(1600));
GO
INSERT INTO TABLE dbo.bigrows VALUES (REPLICATE('a', 1600), '');
INSERT INTO TABLE dbo.bigrows VALUES (REPLICATE('b', 1600), '');
INSERT INTO TABLE dbo.bigrows VALUES (REPLICATE('c', 1600), '');
INSERT INTO TABLE dbo.bigrows VALUES (REPLICATE('d', 1600), '');
INSERT INTO TABLE dbo.bigrows VALUES (REPLICATE('e', 1600), '');
GO
UPDATE TABLE dbo.bigrows
SET c = REPLICATE('x', 1600)
WHERE a = 3;
GO
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
FROM sys.dm_db_database_page_allocations(db_id('AdventureWorks2012'),
    object_id('dbo.bigrows'), null, null, 'DETAILED');
GO

-- Pages 373 - 375
-- LISTING 7-12 Creating a table with a columnstore index
USE AdventureWorksDW2012
GO
IF object_id('dbo.FactInternetSalesBig') IS NOT NULL
  DROP TABLE dbo.FactInternetSalesBig;
GO

CREATE TABLE dbo.FactInternetSalesBig (
  ProductKey int NOT NULL,
  OrderDateKey int NOT NULL,
  DueDateKey int NOT NULL,
  ShipDateKey int NOT NULL,
  CustomerKey int NOT NULL,
  PromotionKey int NOT NULL,
  CurrencyKey int NOT NULL,
  SalesTerritoryKey int NOT NULL,
  SalesOrderNumber nvarchar(20) NOT NULL,
  SalesOrderLineNumber tinyint NOT NULL,
  RevisionNumber tinyint NOT NULL,
  OrderQuantity smallint NOT NULL,
  UnitPrice money NOT NULL,
  ExtendedAmount money NOT NULL,
  UnitPriceDiscountPct float NOT NULL,
  DiscountAmount float NOT NULL,
  ProductStandardCost money NOT NULL,
  TotalProductCost money NOT NULL,
  SalesAmount money NOT NULL,
  TaxAmt money NOT NULL,
  Freight money NOT NULL,
  CarrierTrackingNumber nvarchar(25) NULL,
  CustomerPONumber nvarchar(25) NULL,
  OrderDate datetime NULL,
  DueDate datetime NULL,
  ShipDate datetime NULL
)
GO

INSERT INTO dbo.FactInternetSalesBig
  SELECT * FROM dbo.FactInternetSales;
GO
IF object_id('dbo.RevisionNumberValue') IS NOT NULL
  DROP TABLE dbo.RevisionNumberValue;
GO
CREATE TABLE RevisionNumberValue (RevisionNumber tinyint);
  INSERT INTO RevisionNumberValue SELECT 1;
GO

-- Make sure everything between this comment and the GO 9 is run as a single batch
DECLARE @RevisionNumber tinyint;
SELECT @RevisionNumber = RevisionNumber + 1 FROM RevisionNumberValue;
SELECT @RevisionNumber as RevisionNumber;
INSERT INTO dbo.FactInternetSalesBig WITH (TABLOCK)
SELECT ProductKey
,OrderDateKey
,DueDateKey
,ShipDateKey
,CustomerKey
,PromotionKey
,CurrencyKey
,SalesTerritoryKey
,SalesOrderNumber + cast (@RevisionNumber as nvarchar(4))
,SalesOrderLineNumber
,@RevisionNumber
,OrderQuantity
,UnitPrice
,ExtendedAmount
,UnitPriceDiscountPct
,DiscountAmount
,ProductStandardCost
,TotalProductCost
,SalesAmount
,TaxAmt
,Freight
,CarrierTrackingNumber
,CustomerPONumber
,OrderDate
,DueDate
,ShipDate
FROM dbo.FactInternetSalesBig;
UPDATE RevisionNumberValue SET RevisionNumber = RevisionNumber + 1;
GO 9


SELECT COUNT(*) FROM FactInternetSalesBig;
GO
-- The table will have 30,923,776 rows in it. 
-- Now build the columnstore index on all 26 columns in the table:

CREATE NONCLUSTERED COLUMNSTORE INDEX csi_FactInternetSalesBig
ON dbo.FactInternetSalesBig (
  ProductKey,
  OrderDateKey,
  DueDateKey,
  ShipDateKey,
  CustomerKey,
  PromotionKey,
  CurrencyKey,
  SalesTerritoryKey,
  SalesOrderNumber,
  SalesOrderLineNumber,
  RevisionNumber,
  OrderQuantity,
  UnitPrice,
  ExtendedAmount,
  UnitPriceDiscountPct,
  DiscountAmount,
  ProductStandardCost,
  TotalProductCost,
  SalesAmount,
  TaxAmt,
  Freight,
  CarrierTrackingNumber,
  CustomerPONumber,
  OrderDate,
  DueDate,
  ShipDate
);
GO

-- Page 376
-- LISTING 7-13 Displaying the allocation units for a table with a columnstore index
SELECT index_id, rows, data_compression_desc, type_desc, total_pages
FROM sys.partitions p
JOIN sys.allocation_units au
ON p.partition_id = au.container_id
WHERE OBJECT_NAME(object_id) = 'FactInternetSalesBig';
GO


-- Pages 377 - 378
-- LISTING 7-14 Displaying the columns in the columnstore index
SELECT s.column_id, col_name(ic.object_id, ic.column_id) as column_name,
    count(*) as segment_count
FROM sys.column_store_segments s join sys.partitions p
    ON s.partition_id = p.partition_id
       LEFT JOIN sys.index_columns ic
       ON p.object_id = ic.object_id AND p.index_id = ic.index_id
          AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactInternetSalesBig'
GROUP BY s.column_id, col_name(ic.object_id, ic.column_id)
ORDER by s.column_id;
GO


-- Pages 378 - 379
-- Next is a query that provides the number of rows in each segment, 
-- but because each column has the same number of rows in each segment, 
-- the query is filtered on only a single column:
SELECT segment_id, sum(row_count)
FROM sys.column_store_segments s join sys.partitions p
  ON s.partition_id = p.partition_id
  JOIN sys.index_columns ic
    ON p.object_id = ic.object_id AND p.index_id = ic.index_id
       AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactInternetSalesBig' and index_column_id = 1
GROUP BY segment_id;
GO


-- Page 379
--LISTING 7-15 Displaying information about each segment for one column
SELECT segment_id, min_data_id, max_data_id
FROM sys.column_store_segments s join sys.partitions p
   ON s.partition_id = p.partition_id
   LEFT JOIN sys.index_columns ic
     ON p.object_id = ic.object_id AND p.index_id = ic.index_id
        AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactInternetSalesBig'
      AND col_name(ic.object_id, ic.column_id) = 'OrderDateKey'
ORDER by segment_id;
GO

