-- Chapter 8 "Special storage"


-- Page 383

-- To determine whether SQL Server is storing any data in row-overflow data pages 
-- for a particular table, you can run the following allocation query from Chapter 5 

SELECT object_name(object_id) AS name, partition_id, partition_number AS pnum, rows,
        allocation_unit_id AS au_id, type_desc as page_type_desc, total_pages AS pages
FROM sys.partitions p JOIN sys.allocation_units a
     ON p.partition_id = a.container_id
WHERE object_id=object_id('dbo.bigrows');
 
-- Alternatively, you can use the sys.dm_db_database_page_allocations function and 
-- see the four pages individually:
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
    object_id as ObjectID, partition_id AS PartitionID,
    allocation_unit_type_desc as AU_type, page_type as PageType
FROM sys.dm_db_database_page_allocations
         (db_id('testdb'), object_id('bigrows'), null, null, 'DETAILED');
         
         
-- Pages 388 - 389

-- LISTING 8-1 Storing LOB data on two types of pages
IF OBJECT_ID('textdata') IS NOT NULL  DROP TABLE textdata;
GO
CREATE TABLE textdata (bigcol text);
GO
INSERT INTO textdata
    SELECT REPLICATE(convert(varchar(MAX), 'a'), 38000);
GO
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
    object_id as ObjectID, partition_id AS PartitionID,
    allocation_unit_type_desc as AU_type, page_type as PageType
FROM sys.dm_db_database_page_allocations(db_id('testdb'), object_id('textdata'),
                  null, null, 'DETAILED');
GO
INSERT INTO textdata SELECT REPLICATE(convert(varchar(MAX), 'a'), 41000);
GO
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
    object_id as ObjectID, partition_id AS PartitionID,
    allocation_unit_type_desc as AU_type, page_type as PageType
FROM sys.dm_db_database_page_allocations(db_id('testdb'), object_id('textdata'),
                null, null, 'DETAILED');
GO

-- Page 390

-- LISTING 8-2 Finding basic information for the HasText table
SELECT convert(char(7), object_name(object_id)) AS name, partition_id,
     partition_number AS pnum, rows, allocation_unit_id AS au_id, 
     convert(char(17), type_desc) as page_type_desc, total_pages AS pages
FROM sys.partitions p JOIN sys.allocation_units a
     ON p.partition_id = a.container_id
WHERE object_id=object_id('dbo.HasText');

SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
    object_id as ObjectID, partition_id AS PartitionID, allocation_unit_type_desc as AU_Type,
    page_type as PageType
FROM sys.dm_db_database_page_allocations(db_id('testdb'),
        object_id('textdata'), null, null, 'DETAILED');
GO


--Pages 402 - 405
-- LISTING 8-3 Creating a database to hold filestream data
USE master;
GO
DROP DATABASE MyFilestreamDB;
GO
CREATE DATABASE MyFilestreamDB ON PRIMARY
    (NAME = N'Rowdata1', FILENAME = N'c:\data\Rowdata1.mdf' , SIZE = 2304KB ,
     MAXSIZE = UNLIMITED, FILEGROWTH = 1024KB ),
     FILEGROUP FileStreamGroup1 CONTAINS FILESTREAM DEFAULT( NAME = FSData1,
     FILENAME = 'c:\Data2\FileStreamGroup1'),
     FILEGROUP FileStreamGroup2 CONTAINS FILESTREAM ( NAME = FSData2,
     FILENAME = 'c:\Data2\FileStreamGroup2')
  LOG ON
    (NAME = N'FSDBLOG', FILENAME = N'c:\data\FSDB_log.ldf' , SIZE = 1024KB ,
     MAXSIZE = 2048GB , FILEGROWTH = 10%);
GO
USE MyFilestreamDB;
GO
CREATE TABLE dbo.Records
(
  Id [uniqueidentifier] ROWGUIDCOL NOT NULL UNIQUE,
  SerialNumber INTEGER UNIQUE,
  Chart_Primary VARBINARY(MAX) FILESTREAM NULL
)
FILESTREAM_ON FileStreamGroup1;
GO
INSERT INTO dbo.Records
 VALUES 
   (newid(), 1, 
     CAST (REPLICATE (CONVERT(varchar(MAX), 'Base Data'), 10000) as varbinary(max))),
   (newid(), 2,
     CAST (REPLICATE (CONVERT(varchar(MAX), 'New Data'), 10000) as varbinary(max))),
   (newid(), 3, 0x);
GO
BACKUP DATABASE MyFileStreamDB to disk = 'C:\backups\FBDB.bak';
GO
-- Now delete one of the rows, as follows:
DELETE dbo.Records
WHERE SerialNumber = 2;
GO

-- Page 412

-- The following example shows the creation of two very similar tables: 
-- one that doesnt allow sparse columns and another that does. I attempt 
-- to insert the same rows into each table. Because a row allowing sparse 
-- columns has a smaller maximum length, it fails when trying to insert 
-- a row that the table with no sparse columns has no problem with:
USE testdb;
GO
IF OBJECT_ID('test_nosparse') IS NOT NULL DROP TABLE test_nosparse;
GO
CREATE TABLE test_nosparse
(
 col1 int,
 col2 char(8000),
 col3 varchar(8000)
);
GO
INSERT INTO test_nosparse SELECT null, null, null;
INSERT INTO test_nosparse SELECT 1, 'a', 'b';
GO

-- These two rows can be inserted with no error. 
-- Now, build the second table:
IF OBJECT_ID('test_sparse') IS NOT NULL DROP TABLE test_sparse;
GO
CREATE TABLE test_sparse
(
 col1 int SPARSE,
 col2 char(8000) SPARSE,
 col3 varchar(8000) SPARSE
);
GO
INSERT INTO test_sparse SELECT NULL, NULL, NULL;
INSERT INTO test_sparse SELECT 1, 'a', 'b';
GO

-- Page 413

-- the following code creates a table with large rows, but the INSERT statements, 
-- with or without NULLs, are accepted. However, when you try to make one of 
-- the columns SPARSEeven a relatively small column like the 8-byte datetime column
-- the extra overhead makes the existing rows too large and the ALTER fails:
IF OBJECT_ID('test_nosparse_alter') IS NOT NULL DROP TABLE test_nosparse_alter;
GO
CREATE TABLE test_nosparse_alter
(
 c1 int,
 c2 char(4020) ,
 c3 char(4020) ,
 c4 datetime
);
GO
INSERT INTO test_nosparse_alter SELECT NULL, NULL, NULL, NULL;
INSERT INTO test_nosparse_alter SELECT 1, 1, 'b', GETDATE();
GO
ALTER TABLE test_nosparse_alter
   ALTER COLUMN c4 datetime SPARSE;
GO

-- Pages 414 - 415

LISTING 8-4 Building a table with an identity column, sparse columns, and a column set
USE testdb;
GO
IF EXISTS (SELECT * FROM sys.tables WHERE name = 'lots_of_sparse_columns')
    DROP TABLE lots_of_sparse_columns;
GO
CREATE TABLE lots_of_sparse_columns
(ID int IDENTITY,
 col1 int SPARSE,
 col2 int SPARSE,
 col3 int SPARSE,
 col4 int SPARSE,
 col5 int SPARSE,
 col6 int SPARSE,
 col7 int SPARSE,
 col8 int SPARSE,
 col9 int SPARSE,
 col10 int SPARSE,
 col11 int SPARSE,
 col12 int SPARSE,
 col13 int SPARSE,
 col14 int SPARSE,
 col15 int SPARSE,
 col16 int SPARSE,
 col17 int SPARSE,
 col18 int SPARSE,
 col19 int SPARSE,
 col20 int SPARSE,
 col21 int SPARSE,
 col22 int SPARSE,
 col23 int SPARSE,
 col24 int SPARSE,
 col25 int SPARSE,
 sparse_column_set XML COLUMN_SET FOR ALL_SPARSE_COLUMNS);
GO

-- Page 418

-- Now look at the bytes of a row containing sparse columns. First, build a table 
-- containing two sparse columns, and populate it with three rows:
USE testdb;
GO
IF OBJECT_ID ('sparse_bits') IS NOT NULL DROP TABLE sparse_bits;
GO
CREATE TABLE sparse_bits
(
 c1 int IDENTITY,
 c2 varchar(4),
 c3 char(4) SPARSE,
 c4 varchar(4) SPARSE
);
GO
INSERT INTO sparse_bits SELECT 'aaaa', 'bbbb', 'cccc';
INSERT INTO sparse_bits SELECT 'dddd', null, 'eeee';
INSERT INTO sparse_bits SELECT 'ffff', null, 'gg';
GO
-- Now you can use sys.dm_db_database_page_allocations to find the page number 
-- for the data page storing these three rows and then use DBCC PAGE to look 
-- at the bytes on the page:
SELECT allocated_page_file_id as PageFID, allocated_page_page_id as PagePID,
     object_id as ObjectID, partition_id AS PartitionID,
     allocation_unit_type_desc as AU_type, page_type as PageType
FROM sys.dm_db_database_page_allocations
      (db_id('testdb'), object_id('sparse_bits'), null, null, 'DETAILED');
-- The output indicated that the data page for my table was on page 289;
DBCC TRACEON(3604);
DBCC PAGE(testdb, 1, 289, 1);
GO


-- Pages 420 - 421

-- LISTING 8-5 Saving space with sparse columns
USE testdb;
GO
SET NOCOUNT ON;
GO
IF OBJECT_ID('sparse_nonulls_size') IS NOT NULL DROP TABLE sparse_nonulls_size;
GO
CREATE TABLE sparse_nonulls_size
(col1 int IDENTITY,
 col2 datetime SPARSE,
 col3 char(10) SPARSE
);
GO
IF OBJECT_ID('nonsparse_nonulls_size') IS NOT NULL DROP TABLE nonsparse_nonulls_size;
GO
CREATE TABLE nonsparse_nonulls_size
(col1 int IDENTITY,
 col2 datetime,
 col3 char(10)
);
GO
IF OBJECT_ID('sparse_nulls_size') IS NOT NULL DROP TABLE sparse_nulls_size;
GO
CREATE TABLE sparse_nulls_size
(col1 int IDENTITY,
 col2 datetime SPARSE,
 col3 char(10) SPARSE
);
GO
IF OBJECT_ID('nonsparse_nulls_size') IS NOT NULL DROP TABLE nonsparse_nulls_size;
GO
CREATE TABLE nonsparse_nulls_size
(col1 int IDENTITY,
 col2 datetime,
 col3 char(10)
);
GO
DECLARE @num int
SET @num = 1
WHILE @num < 100000
BEGIN
  INSERT INTO sparse_nonulls_size SELECT GETDATE(), 'my message';
  INSERT INTO nonsparse_nonulls_size SELECT GETDATE(), 'my message';
  INSERT INTO sparse_nulls_size SELECT NULL, NULL;
  INSERT INTO nonsparse_nulls_size SELECT NULL, NULL;
  SET @num = @num + 1;
END;
GO
-- Now look at the number of pages in each table. The following metadata query 
-- looks at the number of data pages in the sys.allocation_units view for 
-- each of the four tables:
SELECT object_name(object_id) as 'table with 100K rows', data_pages
  FROM sys.allocation_units au JOIN sys.partitions p
    ON p.partition_id = au.container_id
WHERE object_name(object_id) LIKE '%sparse%size';
GO

-- Pages 425 - 426

-- LISTING 8-6 Comparing two tables to show row compression
USE AdventureWorks2012;
GO
IF OBJECT_ID('Employees_uncompressed') IS NOT NULL DROP TABLE Employees_uncompressed;
GO
GO
SELECT e.BusinessEntityID, NationalIDNumber, JobTitle, BirthDate, 
    MaritalStatus, VacationHours, FirstName, LastName
  INTO Employees_uncompressed
  FROM HumanResources.Employee e JOIN Person.Person p
    ON e.BusinessEntityID = p.BusinessEntityID;
GO
UPDATE Employees_uncompressed
  SET NationalIDNumber = '1111',
      JobTitle = 'Boss',
      LastName = 'Gato'
  WHERE FirstName = 'Ken'
     AND LastName = 'Snchez';
GO
ALTER TABLE dbo.Employees_uncompressed
  ADD CONSTRAINT EmployeeUn_ID PRIMARY KEY (BusinessEntityID);
GO
SELECT OBJECT_NAME(object_id) as name, rows, data_pages, data_compression_desc
  FROM sys.partitions p JOIN sys.allocation_units au
    ON p.partition_id = au.container_id
  WHERE object_id = object_id('dbo.Employees_uncompressed');
IF OBJECT_ID('Employees_rowcompressed') IS NOT NULL DROP TABLE Employees_rowcompressed;
GO
SELECT BusinessEntityID, NationalIDNumber, JobTitle, BirthDate, 
    MaritalStatus, VacationHours, FirstName, LastName
  INTO Employees_rowcompressed
  FROM dbo.Employees_uncompressed
GO
ALTER TABLE dbo.Employees_rowcompressed ADD CONSTRAINT EmployeeR_ID PRIMARY KEY (BusinessEntityID);
GO
ALTER TABLE dbo.Employees_rowcompressed REBUILD WITH (DATA_COMPRESSION = ROW);
GO
SELECT OBJECT_NAME(object_id) as name, rows, data_pages, data_compression_desc
  FROM sys.partitions p JOIN sys.allocation_units au
    ON p.partition_id = au.container_id
WHERE object_id = object_id('dbo.Employees_rowcompressed');
GO

-- Page 434
-- LISTING 8-7 Applying page compression to a table
USE AdventureWorks2012;
GO
IF OBJECT_ID('Employees_pagecompressed') IS NOT NULL DROP TABLE Employees_pagecompressed;
GO
SELECT BusinessEntityID, NationalIDNumber, JobTitle, BirthDate, 
    MaritalStatus, VacationHours, FirstName, LastName
  INTO Employees_pagecompressed
  FROM dbo.Employees_uncompressed
GO
ALTER TABLE dbo.Employees_pagecompressed ADD CONSTRAINT EmployeeP_ID PRIMARY KEY (BusinessEntityID);
GO
ALTER TABLE dbo.Employees_pagecompressed REBUILD WITH (DATA_COMPRESSION = PAGE);
GO
SELECT OBJECT_NAME(object_id) as name, rows, data_pages, data_compression_desc
  FROM sys.partitions p JOIN sys.allocation_units au
    ON p.partition_id = au.container_id
  WHERE object_id = object_id('dbo.Employees_pagecompressed');
GO
SELECT object_name(object_id) as Table_Name, count(*) as Page_Count
  FROM sys.dm_db_database_page_allocations(db_id('AdventureWorks2012'), null, null, null, 'DETAILED')
  WHERE object_name(object_id) like ('Employees%compressed')
    AND page_type_desc = 'DATA_PAGE'
  GROUP BY object_name(object_id);
GO

-- Pages 447 - 448

--LISTING 8-8 View returning data about each partition of each partitioned table
CREATE FUNCTION dbo.index_name (@object_id int, @index_id tinyint)
RETURNS sysname
AS
  BEGIN
    DECLARE @index_name sysname
    SELECT @index_name = name FROM sys.indexes
    WHERE object_id = @object_id and index_id = @index_id
  RETURN(@index_name)
  END;
GO
CREATE VIEW Partition_Info AS
SELECT OBJECT_NAME(i.object_id) as ObjectName, dbo.INDEX_NAME(i.object_id,i.index_id) AS IndexName,
    object_schema_name(i.object_id) as SchemaName, p.partition_number as PartitionNumber, 
    fg.name AS FilegroupName, rows as Rows, au.total_pages as TotalPages,
   CASE boundary_value_on_right
     WHEN 1 THEN 'less than'
     ELSE 'less than or equal to'
   END as 'Comparison',
    rv.value as BoundaryValue,
   CASE WHEN ISNULL(rv.value, rv2.value) IS NULL THEN 'N/A'
    ELSE 
      CASE 
        WHEN boundary_value_on_right = 0 AND rv2.value IS NULL  
           THEN 'Greater than or equal to'
        WHEN boundary_value_on_right = 0 
           THEN 'Greater than' 
        ELSE 'Greater than or equal to' END + ' ' +
           ISNULL(CONVERT(varchar(20), rv2.value), 'Min Value') 
                + ' ' +
                + 
           CASE boundary_value_on_right 
             WHEN 1 THEN 'and less than' 
               ELSE 'and less than or equal to' 
               END + ' ' +
                + ISNULL(CONVERT(varchar(20), rv.value), 
                           'Max Value')
        END as 'TextComparison'
  FROM sys.partitions p 
    JOIN sys.indexes i 
      ON p.object_id = i.object_id and p.index_id = i.index_id
    LEFT JOIN sys.partition_schemes ps 
      ON ps.data_space_id = i.data_space_id
    LEFT JOIN sys.partition_functions f 
      ON f.function_id = ps.function_id
    LEFT JOIN sys.partition_range_values rv 
      ON f.function_id = rv.function_id 
          AND p.partition_number = rv.boundary_id     
    LEFT JOIN sys.partition_range_values rv2 
      ON f.function_id = rv2.function_id 
          AND p.partition_number - 1= rv2.boundary_id
    LEFT JOIN sys.destination_data_spaces dds
      ON dds.partition_scheme_id = ps.data_space_id 
          AND dds.destination_id = p.partition_number 
    LEFT JOIN sys.filegroups fg 
      ON dds.data_space_id = fg.data_space_id
    JOIN sys.system_internals_allocation_units au
      ON au.container_id = p.partition_id 
WHERE i.index_id <2 AND au.type =1;
GO

-- Page 453
-- First, create a very simple partition function and partition scheme. 
-- The partition function splits the data into five partitions, which eventually 
-- are mapped to the SalesTerritoryKey column of the big table. Then define a 
-- partition scheme that puts all the partitions on the PRIMARY filegroup:
USE AdventureWorksDW2012
GO
CREATE PARTITION FUNCTION [PF_TerritoryKey](int) AS RANGE LEFT FOR VALUES (2, 4, 6, 8)
GO
CREATE PARTITION SCHEME [PS_TerritoryKey] AS PARTITION [PF_TerritoryKey] ALL TO ([PRIMARY]);
GO
-- Now you can rebuild the clustered index to use this partitioning scheme, but you 
-- should get an error message initially, because if a table has a columnstore index,
--  it must be partitioned aligned with the table. So you have to drop the 
-- columnstore index before you can rebuild the clustered index, and then you can 
-- rebuild the columnstore index using the same partitioning scheme.
DROP INDEX dbo.FactInternetSalesBig.csi_FactInternetSalesBig;
GO
CREATE CLUSTERED INDEX clus_FactInternetSalesBig ON dbo.FactInternetSalesBig (SalesTerritoryKey)
ON PS_TerritoryKey (SalesTerritoryKey)
GO

-- Page 453
-- LISTING 8-9 Rebuilding the columnstore index on the same partitioning scheme
CREATE NONCLUSTERED COLUMNSTORE INDEX csi_FactInternetSalesBig
ON dbo.FactInternetSalesBig (
 ProductKey,
 OrderDateKey,
 DueDateKey,
 ShipDateKey,
 CustomerKey,
 PromotionKey,
 CurrencyKey,
 SalesTerritoryKey,
 SalesOrderNumber,
 SalesOrderLineNumber,
 RevisionNumber,
 OrderQuantity,
 UnitPrice,
 ExtendedAmount,
 UnitPriceDiscountPct,
 DiscountAmount,
 ProductStandardCost,
 TotalProductCost,
 SalesAmount,
 TaxAmt,
 Freight,
 CarrierTrackingNumber,
 CustomerPONumber
) ON PS_TerritoryKey (SalesTerritoryKey)
GO

-- Page 454
-- The following query groups by column to show you the total number of segments 
-- in the table. If you run this query, youll see 24 rows indicating 24 columns. 
-- The index had only 23 columns defined, but because the clustered index wasnt 
-- unique, the uniquifier is added as a column. The result of this query also shows 
-- 41 total segments:
-- GROUP BY COLUMN
SELECT s.column_id, col_name(ic.object_id, ic.column_id) as column_name, 
       count(*) as segment_count
FROM sys.column_store_segments s JOIN sys.partitions p 
     ON  s.partition_id = p.partition_id
  LEFT JOIN sys.index_columns ic
    ON p.object_id = ic.object_id AND p.index_id = ic.index_id
       AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactInternetSalesBig'
GROUP BY s.column_id, col_name(ic.object_id, ic.column_id), object_name(p.object_id)
ORDER by 1;
GO
-- Because of the boundary values used, not every partition has exactly the 
-- same number of rows, as you saw in the data from the Partition_Info view. 
-- Each partition could have a different number of segments, and the following 
-- query shows how many segments are created for each partition:
SELECT partition_number, count( segment_id) as NumSegments, sum(row_count) as NumRows
FROM sys.column_store_segments s join sys.partitions p on s.partition_id = p.partition_id
  JOIN sys.index_columns ic
    ON p.object_id = ic.object_id AND p.index_id = ic.index_id
       AND s.column_id = ic.index_column_id
WHERE object_name(p.object_id) = 'FactInternetSalesBig' and index_column_id = 2
GROUP BY partition_number WITH ROLLUP;
GO